"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("tns-core-modules/utils/types");
var useLegacy = false;
var policies = {
    def: AFSecurityPolicy.defaultPolicy(),
    secured: false,
};
policies.def.allowInvalidCertificates = true;
policies.def.validatesDomainName = false;
function enableSSLPinning(options) {
    if (!policies.secure) {
        policies.secure = AFSecurityPolicy.policyWithPinningMode(1);
        policies.secure.allowInvalidCertificates = (types_1.isDefined(options.allowInvalidCertificates)) ? options.allowInvalidCertificates : false;
        policies.secure.validatesDomainName = (types_1.isDefined(options.validatesDomainName)) ? options.validatesDomainName : true;
        var data = NSData.dataWithContentsOfFile(options.certificate);
        policies.secure.pinnedCertificates = NSSet.setWithObject(data);
    }
    useLegacy = (types_1.isDefined(options.useLegacy)) ? options.useLegacy : false;
    policies.secured = true;
    console.log('nativescript-https > Enabled SSL pinning');
}
exports.enableSSLPinning = enableSSLPinning;
function disableSSLPinning() {
    policies.secured = false;
    console.log('nativescript-https > Disabled SSL pinning');
}
exports.disableSSLPinning = disableSSLPinning;
console.info('nativescript-https > Disabled SSL pinning by default');
function AFSuccess(resolve, task, data) {
    var content = getData(data);
    resolve({ task: task, content: content });
}
function AFFailure(resolve, reject, task, error) {
    var data = error.userInfo.valueForKey(AFNetworkingOperationFailingURLResponseDataErrorKey);
    var parsedData = getData(data);
    if (useLegacy) {
        var failure = {
            body: parsedData,
            description: error.description,
            reason: error.localizedDescription,
            url: error.userInfo.objectForKey('NSErrorFailingURLKey').description
        };
        if (policies.secured === true) {
            failure.description = 'nativescript-https > Invalid SSL certificate! ' + error.description;
        }
        var reason = error.localizedDescription;
        resolve({ task: task, content: parsedData, reason: reason, failure: failure });
    }
    else {
        var content = {
            body: parsedData,
            description: error.description,
            reason: error.localizedDescription,
            url: error.userInfo.objectForKey('NSErrorFailingURLKey').description
        };
        if (policies.secured === true) {
            content.description = 'nativescript-https > Invalid SSL certificate! ' + content.description;
        }
        var reason = error.localizedDescription;
        resolve({ task: task, content: content, reason: reason });
    }
}
function request(opts) {
    return new Promise(function (resolve, reject) {
        try {
            var manager_1 = AFHTTPSessionManager.alloc().initWithBaseURL(NSURL.URLWithString(opts.url));
            if (opts.headers && opts.headers['Content-Type'].substring(0, 16) === 'application/json') {
                manager_1.requestSerializer = AFJSONRequestSerializer.serializer();
                manager_1.responseSerializer = AFJSONResponseSerializer.serializerWithReadingOptions(4);
            }
            else {
                manager_1.requestSerializer = AFHTTPRequestSerializer.serializer();
                manager_1.responseSerializer = AFHTTPResponseSerializer.serializer();
            }
            manager_1.requestSerializer.allowsCellularAccess = true;
            manager_1.securityPolicy = (policies.secured === true) ? policies.secure : policies.def;
            var heads_1 = opts.headers;
            if (heads_1) {
                Object.keys(heads_1).forEach(function (key) { return manager_1.requestSerializer.setValueForHTTPHeaderField(heads_1[key], key); });
            }
            var dict_1 = null;
            if (opts.body) {
                var cont_1 = opts.body;
                if (Array.isArray(cont_1)) {
                    dict_1 = NSMutableArray.new();
                    cont_1.forEach(function (item, idx) {
                        dict_1.addObject(item);
                    });
                }
                else if (types_1.isObject(cont_1)) {
                    dict_1 = NSMutableDictionary.new();
                    Object.keys(cont_1).forEach(function (key) { return dict_1.setValueForKey(cont_1[key], key); });
                }
            }
            manager_1.requestSerializer.timeoutInterval = opts.timeout ? opts.timeout : 10;
            var headers = null;
            var success = function (task, data) {
                AFSuccess(resolve, task, data);
            };
            var failure = function (task, error) {
                AFFailure(resolve, reject, task, error);
            };
            var progress = function (progress) {
            };
            if (opts.method === "GET") {
                manager_1.GETParametersHeadersProgressSuccessFailure(opts.url, dict_1, headers, progress, success, failure);
            }
            else if (opts.method === "POST") {
                manager_1.POSTParametersHeadersProgressSuccessFailure(opts.url, dict_1, headers, progress, success, failure);
            }
            else if (opts.method === "PUT") {
                manager_1.PUTParametersHeadersSuccessFailure(opts.url, dict_1, headers, success, failure);
            }
            else if (opts.method === "DELETE") {
                manager_1.DELETEParametersHeadersSuccessFailure(opts.url, dict_1, headers, success, failure);
            }
            else if (opts.method === "PATCH") {
                manager_1.PATCHParametersHeadersSuccessFailure(opts.url, dict_1, headers, success, failure);
            }
            else if (opts.method === "HEAD") {
                manager_1.HEADParametersHeadersSuccessFailure(opts.url, dict_1, headers, success, failure);
            }
        }
        catch (error) {
            reject(error);
        }
    }).then(function (AFResponse) {
        var sendi = {
            content: AFResponse.content,
            headers: {},
        };
        var response = AFResponse.task.response;
        if (!types_1.isNullOrUndefined(response)) {
            sendi.statusCode = response.statusCode;
            var dict = response.allHeaderFields;
            dict.enumerateKeysAndObjectsUsingBlock(function (k, v) { return sendi.headers[k] = v; });
        }
        if (AFResponse.reason) {
            sendi.reason = AFResponse.reason;
        }
        return Promise.resolve(sendi);
    });
}
exports.request = request;
function getData(data) {
    var content;
    if (data && data.class) {
        if (data.enumerateKeysAndObjectsUsingBlock || data instanceof NSArray) {
            var serial = NSJSONSerialization.dataWithJSONObjectOptionsError(data, 1);
            content = NSString.alloc().initWithDataEncoding(serial, NSUTF8StringEncoding).toString();
        }
        else if (data instanceof NSData) {
            content = NSString.alloc().initWithDataEncoding(data, NSASCIIStringEncoding).toString();
        }
        else {
            content = data;
        }
        try {
            content = JSON.parse(content);
        }
        catch (ignore) {
        }
    }
    else {
        content = data;
    }
    return content;
}
//# sourceMappingURL=https.ios.js.map